package EgadsConfig;
# Package to read config/remote/Slave.cfg file to extract Ports and Dirs
# hashes for use by Data Collector packages.

use Debug qw(debug addToSumFile);
use strict;
use Exporter;
use XML::Parser;
use Data::Dumper;
use POSIX qw(strftime);

use vars qw(@ISA $VERSION);

@ISA = qw(Exporter);

my $VERSION = do {my @r=(q$Revision: 1.1 $=~/\d+/g);sprintf "%d."."%02d"x$#r,@r};
my %configs;
my %filenames;

sub new {
	my $class = shift;
	my $file = shift;
	my $self = {};

	debug(50,"EgadsConfig::new: Entering...\n");
	# Load the file.
	my $p1 = new XML::Parser(Style => 'Subs');
	$p1->parsefile($file) or do {
		$@ = "EGADS::EgadsConfig cannot open `$file' for reading: $!\n";
		debug(1,"EgadsConfig::new: $@...\n");
		return;
	};

	$self->{configs} = { %configs };
	$self->{filenames} = { %filenames };
	undef(%configs);
	undef(%filenames);
	debug(50,"EgadsConfig::new: Exiting...\n");
	bless $self, $class;
	$self;
}

sub config {
	my ($xp, $el, %att) = @_;
	$configs{$att{id}} = [] unless ($configs{$att{id}});
	push(@{$configs{$att{id}}}, $att{value});
}

sub filenames {
	my ($xp, $el, %att) = @_;
	my $make  = lc($att{make});
	my $model = lc($att{model});
	my $ext   = $att{extension};
	my $name  = $att{filename};
	$filenames{$make} = {} unless ($filenames{$make});
	$filenames{$make}->{$model} = {
		extension => $ext,
		filename => $name
	};
}

sub getValue {
	my $self = shift;
	my $id = shift;
	return(wantarray ? @{$self->{configs}->{$id}} : $self->{configs}->{$id}->[0]) if ($self->{configs}->{$id});
	return(undef);
}

sub getFileName {
	my $self = shift;
	my $make = lc(shift);
	my $model = lc(shift);
	my $extension = (@_) ? shift : "";

	if ($extension ne "") {
		foreach my $make (keys(%{$self->{filenames}})) {
			foreach my $model (keys(%{$self->{filenames}->{$make}})) {
				if ($self->{filenames}->{$make}->{$model}->{extension} eq $extension) {
					return($self->{filenames}->{$make}->{$model}->{filename});
				}
			}
		}
	} else {
		if ($self->{filenames}->{$make}->{$model}->{filename}) {
			return($self->{filenames}->{$make}->{$model}->{filename});
		}
		if ($self->{filenames}->{$make}->{all}->{filename}) {
			return($self->{filenames}->{$make}->{all}->{filename});
		}
	}
	return(undef);
}

sub getExtension {
	my $self = shift;
	my $make = lc(shift);
	my $model = lc(shift);

	if ($self->{filenames}->{$make}->{$model}->{extension}) {
		return($self->{filenames}->{$make}->{$model}->{extension});
	}
	if ($self->{filenames}->{$make}->{all}->{extension}) {
		return($self->{filenames}->{$make}->{all}->{extension});
	}
	return(undef);
}

sub getFileTemplate {
	my $self = shift;
	my $hub = shift;
	my $site = shift;
	my $make = shift;
	my $model = shift;

	my $filetemplate = $self->getValue("filetemplate");
	my $ext = $self->getExtension($make,$model);

	return($self->replacefields($filetemplate,$hub,$site,"",$ext));

}

sub MakeFileName {
	my $self = shift;
	my $site = shift;
	my $start = shift;
	my $stop = shift;
	my $filetemplate = shift;

#get some values from the file template structure that are not passed
#back from client
#	hub
#	session
#	extension

	$_ = $filetemplate;
	my ($extension, $session, $hub);
	my ($gps_week, $gps_sec);
	if (/(\S+)_(\S+)_(\d+)_(\d+)_(\d+)_(\d+)_(\S)\.(\S+)/) {
		my $t_site = $1;
		my $t_hub  = $2;
		my $t_gwk  = $3;
		my $t_gsec = $4;
		my $t_yr   = $5;
		my $t_doy  = $6;
		my $t_sess = $7;
		my $t_ext  = $8;
		$hub = $t_hub;
		$session = $t_sess;
		$extension = $t_ext;
		$gps_week = sprintf("%04d",$t_gwk);
		$gps_sec = sprintf("%06d",$t_gsec);
	}

	my $secinday = sprintf("%05d",$start % 86400);
	my $newfile = $self->getFileName("","",$extension);
	$newfile = $self->replacefields($newfile,$hub,$site,$session,$extension);

	$newfile = strftime($newfile,gmtime($start));
	$newfile =~ s/\+J/$secinday/;
	$newfile =~ s/\+G/$gps_week/;
	$newfile =~ s/\+g/$gps_sec/;

	return($newfile);
	
}

sub replacefields {
	my $self = shift;
	my $filename = shift;
	my $hub = shift;
	my $site = shift;
	my $session = shift;
	my $ext = shift;

	$_ = $filename;

	my ($newhub,$other) = split("-",$hub);
	$newhub = substr($newhub,0,5) if (length($newhub) > 5);;
	$session = substr($session,0,1);
	$site = substr($site,0,4);

	s/%session%/$session/g;
	s/%site%/$site/g;
	s/%hub%/$newhub/g;
	s/%extension%/$ext/g;

	return($_);
}

1;

__END__

