package Filer;

BEGIN {
push(@INC,qw(/usr/lib/perl5/5.6.0 /usr/lib/perl5/site_perl/5.6.0));
use strict;
use Digest::MD5;
use Debug qw(debug addToSumFile);
use EgadsConfig;
use Data::Dumper;
use POSIX qw(strftime);
use File::Path;
use File::Spec::Functions;

}
if (!defined($ENV{EGADSCONFIG})) {
	die "The EGADSCONFIG environment variable is not defined";
}

my $configfile = $ENV{EGADSCONFIG};

my $CONFIG = EgadsConfig->new($configfile);
unless ($CONFIG) {
	die "Problem reading Configuration file $configfile\n";
}


## Available methods
##   savefile

##
## This should be passed the following:
##  site code
##  start time
##  stop time
##  fileinfo {
##    filename
##    filesize
##    md5 sum
##    file data
## }


## good = return(0)
## bad = return(1)

sub new {
	my $class = shift;
	my $site  = shift;
	my $start = shift;
	my $stop  = shift;
	my $info  = shift;

	my $self = { 
		site => $site,
		start => $start,
		stop => $stop,
		info => $info,
		filename => "",
		case => "",
		rawdirs => [],
		dirs => [],
	};

	
	bless($self, $class);

	debug(30, "entry: $self->{info}->{filename} ");
	debug(30, "$self->{info}->{size}=".length($self->{info}->{data})." ");
	debug(30, "$self->{info}->{md5sum}\n");

	my @rawdir = $CONFIG->getValue("rawdir");
	foreach my $dir (@rawdir) {
		$dir = strftime($dir, gmtime($self->{start}));
		push(@{$self->{rawdirs}}, $dir);
	}

	my @datadir = $CONFIG->getValue("datadir");
	foreach my $dir (@datadir) {
		$dir = strftime($dir, gmtime($self->{start}));
		push(@{$self->{dirs}}, $dir);
	}

	my $tmpcase = $CONFIG->getValue("case");
	$self->{case} = 0;
	$self->{case} = -1 if ($tmpcase =~ /lower/i);
	$self->{case} = 1 if ($tmpcase =~ /upper/i);
	debug(40, "\tdirectories:\n\t\t" . join("\n\t\t", @{$self->{dirs}},@{$self->{rawdirs}}) . "\n");

	debug(20, "\tCreating new File Name: ");
	$self->{filename} = $CONFIG->MakeFileName($site,$start,$stop,$self->{info}->{filename});
	$self->{filename} = uc($self->{filename}) if ($self->{case} > 0);
	$self->{filename} = lc($self->{filename}) if ($self->{case} < 0);
	if ($self->{filename} eq "") {
		debug(20, "NOT OK\n");
		return(1);
	} else {
		debug(20, $self->{filename} . "\n");
	}

	debug(20, "\tChecking Size: ");
	debug(20, $self->{info}->{size} . "=" . length($self->{info}->{data}) . " ");
	if ($self->{info}->{size} == length($self->{info}->{data})) {
		debug(20, "OK\n");
	} else {
		debug(20, "NOT OK\n");
		return(1);
	}

	my $ctx = Digest::MD5->new;
	$ctx->add($self->{info}->{data});
	my $md5sum = $ctx->hexdigest;
	debug(20, "\tMD5 SUM: $self->{info}->{md5sum} = $md5sum ");
	if ($self->{info}->{md5sum} eq $md5sum) {
		debug(20, " OK\n");
	} else {
		debug(20, " NOT OK\n");
		return(1);
	}

	debug(20, "\tMake Directories: " . join(" ", @{$self->{dirs}},@{$self->{rawdirs}}) . " ");
	my $result = $self->makeoutputdir();
	if ($result == 0) {
		debug(20, " SUCCESS\n");
	} else {
		debug(20, " FAILED\n");
		return(1);
	}

	debug(20, "\tWrite File: $self->{filename}");
	$result = $self->writefile();
	if ($result == 0) {
		debug(20, "SUCCESS\n");
	} else {
		debug(20, "FAILED\n");
		return(1);
	}

	return(1) unless ($self->forkfilehandler() == 0);
	debug(20, "\tFork Handler\n");

	return(0);

}

sub makeoutputdir() {
	my $self = shift;

	my @alldirs;
	push(@alldirs, @{$self->{dirs}});
	push(@alldirs, @{$self->{rawdirs}});
	foreach my $dir (@alldirs) {
		stat($dir);
		if (-d _ && -w _) {
			;
		} else {
			my @paths = mkpath($dir,0,0775);
			foreach (@paths) {
				return(0) if ($_ eq $dir);
			}
			return(1);
		}
	}
	return(0);
}
	
sub writefile {
	my $self = shift;

	foreach my $dir (@{$self->{rawdirs}}) {
		my $output = catfile($dir,$self->{info}->{filename});
		my $tmp = $output;
		my $i = 1;
		while (-f $tmp) {
			$tmp = $output . "." . $i;
			$i++;
		}
		$output = $tmp;
		open(FH,">$output") or return(1);
		binmode(FH);
		print FH $self->{info}->{data};
		close(FH);
		my @info = stat($output);
		return(1) unless ($info[7] == $self->{info}->{size});
	}
	
	foreach my $dir (@{$self->{dirs}}) {
		my $output = catfile($dir,$self->{filename});
		my $tmp = $output;
		my $i = 1;
		while (-f $tmp) {
			$tmp = $output . "." . $i;
			$i++;
		}
		$output = $tmp;
		open(FH,">$output") or return(1);
		binmode(FH);
		print FH $self->{info}->{data};
		close(FH);
		my @info = stat($output);
		return(1) unless ($info[7] == $self->{info}->{size});
	}

	return(0);

}

sub forkfilehandler {
	my $self = shift;

	my $dir = ${$self->{dirs}}[0];

	
	my $file = $self->{filename};
#	$_ = $self->{info}->{filename};
	
#	my($site, $gwk, $gsec, $yr, $day, $sess);
#	if (/(\S{4})_(\d{4})_(\d{6)_(\d{2})_(\d{3})_(\S).raw/) {
#		$site = $1;
#		$gwk = $2;
#		$gsec = $3;
#		$yr = $4;
#		$day = $5;
#		$sess = $6;
#	}
		
#	my $forkbin = $CONFIG->getValue("fork-bin");
#	print "\tRunning pqinsert (interim) $dir $file";
#	system("$forkbin $dir $file");
#	print " -- done\n";
	return(0);

}

