package MasterSites;

use Data::Dumper;
use strict;
use Exporter;
use XML::Parser;
use GPSSite;

use vars qw(@ISA $VERSION);

@ISA = qw(Exporter);

$VERSION = do {my @r=(q$Revision: 1.1 $=~/\d+/g);sprintf "%d."."%02d"x$#r,@r};

my $currentsite = "";
my @currentblock;
my $sites = {};

sub new {
	my $class = shift;
	my $file  = shift;

	my $self = bless {}, $class;

	# Load the file.
	my $p1 = new XML::Parser(Style => 'Subs');
	my $tree = $p1->parsefile($file) or do {
		$@ = "EGADS::MasterSites cannot open `$file' for reading: $!\n";
		return;
	};

	$self->{file} = $file;
	$self->{sites} = $sites;

	foreach my $key (keys(%{$sites})) {
		$self->{sites}->{$key} = GPSSite->new($sites->{$key});
	}
			
	$self;
}

sub siteInfo {
	my $self = shift;
	my $site = shift;

	return $self->{sites}->{$site};

}

sub releaseLocks {
	my $self = shift;
	my $client = shift;

	foreach my $site (keys(%{$self->{sites}})) {
		if ($self->{sites}->{$site}->isLockedBy($client)) {
			$self->{sites}->{$site}->unlock();
		}
	}
}


sub getSites {
	my $self = shift;

	my $telemetry = (@_) ? shift : "";
	my $telem_spec = (@_) ? shift : "";
	my @return;

	if (($telemetry eq "") || ($telem_spec eq "")) {
		return keys(%{$self->{sites}});
	}

	foreach my $site (keys(%{$self->{sites}})) {
		my $client_info = $self->{sites}->{$site}->getComm();
		if ($telemetry eq $client_info->{telem} &&
			$telem_spec eq $client_info->{telem_spec}) {
			push(@return,$site);
		}
	}
	@return;
}

sub site {
	my ($xp, $el, %att) = @_;
	die "In the middle of site block" if ($currentsite ne "");
	die "Site already loaded"         if ($sites->{$att{code}});

	$currentsite = $att{code};
	$sites->{$currentsite}->{code} = $att{code};
	$sites->{$currentsite}->{name} = $att{name};
	$sites->{$currentsite}->{active} = $att{active};
	1;
}

sub site_ {
	my ($xp, $el, %att) = @_;
	$currentsite = "";

}

sub receiverinfo {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");

	$sites->{$currentsite}->{receiverinfo}->{type} = $att{type};
	$sites->{$currentsite}->{receiverinfo}->{model} = $att{model};
	push(@currentblock,"receiverinfo");
	
	1;
}

sub antenna {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in receiver block" if ($currentblock[-1] eq "receiverinfo");

	$sites->{$currentsite}->{receiverinfo}->{antenna}->{number} = $att{number};
	$sites->{$currentsite}->{receiverinfo}->{antenna}->{type}   = $att{type};
	$sites->{$currentsite}->{receiverinfo}->{antenna}->{dome}   = $att{dome};

	1;
}

sub receiver {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in receiver block" if ($currentblock[-1] eq "receiverinfo");

	$sites->{$currentsite}->{receiverinfo}->{receiver}->{number} = $att{number};
	$sites->{$currentsite}->{receiverinfo}->{receiver}->{type}   = $att{type};

	1;
}

sub antenna_offset {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in receiver block" if ($currentblock[-1] eq "receiverinfo");

	$sites->{$currentsite}->{receiverinfo}->{antenna_offset}->{north} = $att{north};
	$sites->{$currentsite}->{receiverinfo}->{antenna_offset}->{east}   = $att{east};
	$sites->{$currentsite}->{receiverinfo}->{antenna_offset}->{up}   = $att{up};

	1;
}

sub receiverinfo_ {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in receiver block" if ($currentblock[-1] eq "receiverinfo");
	
	pop(@currentblock);
	1;
}

#sub metinfo {
#	my ($xp, $el, %att) = @_;
#	die "Not in site block" if ($currentsite eq "");
#	$sites->{$currentsite}->{metinfo}->{active} = $att{active};
#	$sites->{$currentsite}->{metinfo}->{type}   = $att{type};
#}

#sub siteinfo {
#	my ($xp, $el, %att) = @_;
#	die "Not in site block" if ($currentsite eq "");
#
#	push(@currentblock, "siteinfo");
#	1;
#}

#sub approx_pos {
#	my ($xp, $el, %att) = @_;
#	die "Not in site block" if ($currentsite eq "");
#	die "Not in siteinfo block" if ($currentblock[-1] ne "siteinfo");
#
#
#	my $tmp = {};
#	$tmp->{type} = $att{type};
#	$tmp->{value1} = $att{value1};
#	$tmp->{value2} = $att{value2};
#	$tmp->{value3} = $att{value3};
#
#	push(@{$sites->{$currentsite}->{siteinfo}->{approx_pos}},$tmp);
#
#	1;
#}
#
#sub domes {
#	my ($xp, $el, %att) = @_;
#	die "Not in site block" if ($currentsite eq "");
#	die "Not in siteinfo block" if ($currentblock[-1] ne "siteinfo");
#
#	$sites->{$currentsite}->{siteinfo}->{domes}->{number} = $att{number};
#
#	1;
#}
#
#sub install {
#	my ($xp, $el, %att) = @_;
#	die "Not in site block" if ($currentsite eq "");
#	die "Not in siteinfo block" if ($currentblock[-1] ne "siteinfo");
#
#	$sites->{$currentsite}->{siteinfo}->{install}->{date} = $att{date};
#
#	1;
#
#}
#
#sub siteinfo_ {
#	my ($xp, $el, %att) = @_;
#	die "Not in site block" if ($currentsite eq "");
#	die "Not in siteinfo block" if ($currentblock[-1] ne "siteinfo");
#	
#	pop(@currentblock);
#	1;
#}
#
sub comm {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");

	push(@currentblock, "comm");
	$sites->{$currentsite}->{comm}->{telemetry}  = $att{telemetry};
	$sites->{$currentsite}->{comm}->{telem_spec} = $att{telem_spec};
	$sites->{$currentsite}->{comm}->{type}       = $att{type};
	$sites->{$currentsite}->{comm}->{baud}       = $att{baud};
	1;
}

#sub req_cmnds {
#	my ($xp, $el, %att) = @_;
#	die "Not in site block" if ($currentsite eq "");
#	die "Not in comm block" if ($currentblock[-1] ne "comm");
#
#	$sites->{$currentsite}->{comm}->{req_cmnds} = $att{commands};
#
#	1;
#}

sub phone {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in comm block" if ($currentblock[-1] ne "comm");

#	$sites->{$currentsite}->{comm}->{phone}->{type}     = $att{type};
	$sites->{$currentsite}->{comm}->{phone}->{areacode} = $att{areacode};
	$sites->{$currentsite}->{comm}->{phone}->{number}   = $att{number};
	print "$currentsite = $att{number}\n";
	1;
}

sub freewave {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in comm block" if ($currentblock[-1] ne "comm");

	$sites->{$currentsite}->{comm}->{freewave}->{slave}     = $att{slave};
	$sites->{$currentsite}->{comm}->{freewave}->{repeater1} = $att{repeater1};
	$sites->{$currentsite}->{comm}->{freewave}->{repeater2} = $att{repeater2};
	$sites->{$currentsite}->{comm}->{freewave}->{repeater}  = $att{repeater};
	1;
}

sub allowedtimes {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in comm block" if ($currentblock[-1] ne "comm");

	push(@currentblock, "allowedtimes");
	1;
}

sub time {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in allowedtimes block" if ($currentblock[-1] ne "allowedtimes");

	my $tmp = {};
	my ($hour, $min, $sec) = split(":",$att{start});
	$tmp->{start}  = ($hour*3600) + ($min*60) + $sec;
	($hour, $min, $sec) = split(":",$att{stop});
	$tmp->{stop}  = ($hour*3600) + ($min*60) + $sec;
	push(@{$sites->{$currentsite}->{allowedtimes}}, $tmp);

	1;
}

sub allowedtimes_ {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in allowedtimes block" if ($currentblock[-1] ne "allowedtimes");
	
	pop(@currentblock);
	1;
}


sub comm_ {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in comm block" if ($currentblock[-1] ne "comm");
	
	pop(@currentblock);
	1;
}

sub sessions {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	
	$sites->{$currentsite}->{sessions}->{type}    = $att{type};
	$sites->{$currentsite}->{sessions}->{seconds} = eval $att{seconds};
	$sites->{$currentsite}->{sessions}->{sample}  = $att{sample};
	$sites->{$currentsite}->{sessions}->{minsv}   = $att{minsv};
	$sites->{$currentsite}->{sessions}->{mask}    = $att{mask};

	push(@currentblock, "sessions");
	1;
}

sub session {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in sessions block" if ($currentblock[-1] ne "sessions");
	
	my $tmp = {};
	my $id     = $att{id};
	my ($hour, $min, $sec) = split(":",$att{start});
	$tmp->{start}  = ($hour*3600) + ($min*60) + $sec;
	($hour, $min, $sec) = split(":",$att{stop});
	$tmp->{stop}  = ($hour*3600) + ($min*60) + $sec;
	$tmp->{sample} = $att{sample};
	$tmp->{mask}   = $att{mask};
	$tmp->{minsv}  = $att{minsv};
	$sites->{$currentsite}->{sessions}->{$id} = $tmp;

	1;
}

sub sessions_ {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	die "Not in sessions block" if ($currentblock[-1] ne "sessions");
	
	pop(@currentblock);
	1;
}

sub delete {
	my ($xp, $el, %att) = @_;
	die "Not in site block" if ($currentsite eq "");
	$sites->{$currentsite}->{delete}->{time} = $att{time};

	1;
}

#sub timetodownload {
#	my ($xp, $el, %att) = @_;
#	die "Not in site block" if ($currentsite eq "");
#
#	my $tmp = {};
#	$tmp->{start}  = $att{start};
#	$tmp->{stop}   = $att{stop};
#	push(@{$sites->{$currentsite}->{timetodownload}->{time}}, $tmp);
#
#	1;
#}
#
#sub agency {
#	my ($xp, $el, %att) = @_;
#	die "Not in site block" if ($currentsite eq "");
#
#	$sites->{$currentsite}->{agency}->{name} = $att{name};
#	1;
#}


1;

__END__
