#This should be in the same namespace as the normal Tasks package

package Tasks::Action;

use strict;
use Data::Dumper;

use Debug qw(debug addToSumFile);
use vars qw($AUTOLOAD @ISA);
@ISA = qw(Tasks);

sub new {
	my $self = shift;
	my $class = ref($self) || $self;
	return $self if ref $self;
	
	$self = { @_ };
	if ($self->{args} eq "") {
		$self->{args} = {};
	} else {
		$self->{args} = { eval $self->{args} };
	}
	return bless $self => $class;
}

sub isSpanOK {
	my $self = shift;
	my $start = shift;
	my $stop = shift;
	
	my $dataDown = ${$self->{DB}}->foundInSpans(
			$self->{sitecode},
			$self->{taskid},
			$start,
			$stop);
	debug(130,"MyTasks::isSpanOK: dataDown = $dataDown\n");
	return($dataDown ? 0 : 1);
}

sub getArguments {
	my $self = shift;

	return($self->getTaskSpans($self->{sitecode}, @_));
}

sub returnVal {
	my $self = shift;
	my @results = @_;

	foreach my $job (@results) {
		if ($job->{status} eq "success") {
			# Check for files
			${$self->{DB}}->addSpan($self->{sitecode}, $self->{taskid},
				$job->{start_time},
				$job->{stop_time});
		}
	}
}

sub editSpans {
	my $self = shift;
	my $spans = shift;

	return($spans);
}

package Tasks::Action::download;

use vars qw($AUTOLOAD @ISA);
@ISA = qw(Tasks::Action);
use Data::Dumper;

sub editSpans {
	my $self = shift;
	my $spans = shift;

	return($spans) unless ($#{$spans} >= 0);
	if (defined($self->{args}->{session})) {
		for (my $i = 0; $i <= $#{$spans}; $i++) {
			${$spans}[$i]->{session} = $self->{args}->{session};
		}
	
	}
	my $minstart = ${$spans}[0]->{start_time};
	for (my $i = 0; $i <= $#{$spans}; $i++) {
		my $start = ${$spans}[$i]->{start_time};
		$minstart = $start if ($start < $minstart);
	}
	my $newspans = ${$self->{DB}}->findOpenSpans(
			$self->{sitecode},
			$self->{taskid},
			(3657*24*60*60), $minstart - 1
			);
	for (my $i = 0; $i <= $#{$newspans}; $i++) {
		${$newspans}[$i]->{session} = "z";
	}
	push(@{$spans},@{$newspans});
	return($spans);
}

package Tasks::Action::delete;

use Data::Dumper;

use vars qw($AUTOLOAD @ISA);
@ISA = qw(Tasks::Action);
sub isSpanOK {
	my $self = shift;
	my $start = shift;
	my $stop = shift;

	my $needDel = $self->SUPER::isSpanOK($start,$stop);
	return(0) unless ($needDel);

	unless (defined($self->{args}->{download_id})) {
		debu(-10, "NO ASSOCIATED DOWNLOAD ID\n");
		debu(-10, "ALLOWING DELETION!!!!\n");
		return(1);
	} else {
		my $id = $self->{args}->{download_id};
		my $dataDown = ${$self->{DB}}->foundInSpans(
				$self->{sitecode},
				$id,
				$start,
				$stop);
		return($dataDown? 1 : 0);
	}
}

sub editSpans {
	my $self = shift;
	my $spans = shift;

	return(${$self->{DB}}->collapseSpans($spans));
}

package Tasks::Action::configure;

use vars qw($AUTOLOAD @ISA);
@ISA = qw(Tasks::Action);

sub isSpanOK {
	return(0);
}

sub editSpans {
	return(undef);
}

1;

