package Server;

#BEGIN {

	push(@INC,qw(/usr/lib/perl5/5.6.0 /usr/lib/perl5/site_perl/5.6.0));
	use strict;
	use Debug qw(debug addToSumFile);
	use EgadsConfig;
	use MasterSites;
	use Client;
	use Data::Dumper;
	use Tasks;
	use DB_EGADS;
	$|=1;

	if (!defined($ENV{EGADSCONFIG})) {
		die "The EGADSCONFIG environment variable is not defined";
	}

	my $configfile = $ENV{EGADSCONFIG};

	my $CONFIG = EgadsConfig->new($configfile);
	unless ($CONFIG) {
		die "Problem reading Configuration file $configfile\n";
	}
	
	my $configdir       = $CONFIG->getValue("config_dir");
	$configdir .= "/" unless ($configdir =~ /\/$/);
	my $mastersitesfile = $configdir . $CONFIG->getValue("mastersites");
	my $tasksfile       = $configdir . $CONFIG->getValue("tasks");
	my $clientsfile     = $configdir . $CONFIG->getValue("clients");
	my $databasefile    = $configdir . $CONFIG->getValue("dbname");
	my $WAITTIME        = $CONFIG->getValue("waittime");
	
	debug(5,"Reading MasterSites\n");
	my $MasterSites = MasterSites->new($mastersitesfile);

	debug(5,"Reading Tasks to do\n");
	my $Tasks = Tasks->new($tasksfile);

	debug(5,"Reading Client Configurations\n");
	my $Client = Client->new($clientsfile);
	
	debug(5,"Opening Database\n");
	my $DB = DB_EGADS->new($databasefile);

#	my $test = $DB->findOpenSpans("copr-0",0,time());
#	print Dumper($test);
#	sleep(30);

#}
	
	
sub new {
	my $self = shift;
	my $class = ref($self) || $self;
	my $client = shift or die "Need a hub name\n";

	if ($Client->isClient($client)) {
		bless {
			client => $client
		} => $class;
	} else {
		die "Not a valid client\n";
	}
}

sub setupJob {
	my $self = shift;

	debug(50,"Server::setupJob: Entering...\n");
	$self->{client_info} = $Client->getPort($self->{client});
	$self->{site} = "";

	my @ret = $MasterSites->getSites($self->{client_info}->{telem}, $self->{client_info}->{telem_spec});
	my $sitespan;
	my $locktime;
	foreach my $site (@ret) {
		unless ($Tasks->isActiveTask($site)) {
			debug(25,"Server::setupJob: $site is NOT active\n");
			next;
		}
		if ($MasterSites->siteInfo($site)->isLocked()) {
			debug(25,"Server::setupJob: $site is locked\n");
			next;
		}
		if (! $MasterSites->siteInfo($site)->isAllowedTime()) {
			debug(25,"Server::setupJob: $site Cannot be run at this time\n");
			next;
		}
		unless ($Tasks->isTasks($site) > 0) {
			debug(25,"Server::setupJob: $site has no tasks\n");
			$MasterSites->siteInfo($site)->lock("system", 1200);
			next;
		}
		$locktime = $Tasks->createSpans($site, $DB);
		if ($locktime <= 0) {
			debug(25,"Server::setupJob: createSpans returned no new spans for $site\n");
			$MasterSites->siteInfo($site)->lock("system", 60);
			next;
		}
		$self->{site} = $site;
		last;
	}

	if ($self->{site} ne "") {
		debug(5,"Working on $self->{site}\n");

		if ($MasterSites->siteInfo(
				$self->{site})->lock($self->{client},
				$locktime)) {
			debug(5,"$self->{site} is locked\n");
		}
	}

	debug(50,"Server::setupJob: Exiting...\n");

	return(($self->{site} eq "") ? $WAITTIME : 0);
}

#Call when a job is finished
#Given:
#  retval
#    task1
#      task1 return values
#    task2
#      task2 return values
#    task3, etc
#
sub releaseJob {
	my $self = shift;
	my $retval = shift;

	foreach my $type (keys(%{$retval})) {
		next unless ($retval->{$type});
		$Tasks->returnVal($self->{site}, $DB, $type, @{$retval->{$type}});
#		my $action = $type;
#		$action =~ tr/A-Z/a-z/;
#		my $taskid = $Tasks->getTaskID($self->{site}, $action);
#		my $class = "Tasks::Action::$action";
#		my $tasktype = $class->new(
#				DB => \$DB,
#				sitecode => $self->{site},
#				taskid => $taskid,
#				args => "",
#				Tasks => \$Tasks);
#		$tasktype->returnVal(@{$retval->{$type}});
	}

	$self->releaseLocks();
#If there are still spans that need to be finished, lock the site for some amount 
# of time.
	$MasterSites->siteInfo($self->{site})->lock("system", 1800);
}

sub releaseLocks {
	my $self = shift;
	debug(25,"Releasing locks for $self->{client}\n");
	$MasterSites->releaseLocks($self->{client});
}

sub getReportInfo {
	my $self = shift;
	my $start = shift;
	my $stop = shift;
	my @sites;
	@_ = $MasterSites->getSites() unless ($#_ >= 0);
	push(@sites,@_);

	my $return = {};
	foreach my $site (@sites) {
		next if ($Tasks->isTasks($site) <= 0);
		debug(25,"Getting report info for $site\n");
		my $taskid = $Tasks->getTaskID($site, "download");
		$return->{$site} = $DB->findOpenSpans($site,$taskid,$start,$stop);
	}
	return($return);
}

sub getLockInfo {
	my $self = shift;
	my @sites;
	@_ = $MasterSites->getSites() unless ($#_ >= 0);
	push(@sites,@_);

	my $return = {};
	foreach my $site (@sites) {
		$return->{$site} = $MasterSites->siteInfo($site)->getLockClient();
	}
	return($return);
}

sub getTaskInfo {
	my $self = shift;
	my $tasktype = shift;
	my @sites;
	@_ = $MasterSites->getSites() unless ($#_ >= 0);
	push(@sites,@_);

	my $return = {};
	foreach my $site (@sites) {
		if ($Tasks->isActiveTask($site) == 0) {
			$return->{$site} = 0;
			next;
		}
		next unless ($Tasks->isTasks($site) > 0);
		my $taskid = $Tasks->getTaskID($site, $tasktype);
		$return->{$site} = $Tasks->getSiteTaskbyID($site,$taskid);
	}
	return($return);
}

# Given
#
#site code
#status_message (text or HTML)
#
sub jobStatus {
	my $self = shift;

}

sub getSiteCode {
	my $self = shift;
	return($self->{site});
}

sub getPortInfo {
	my $self = shift;
	my $port = {
		'device' => $self->{client_info}->{device},
		'baud' => $self->{client_info}->{baud},
		'prefix' => $self->{client_info}->{prefix},
		'initstring' => $self->{client_info}->{initstring}
	};
	return($port);
}

sub getSiteInfo {
	my $self = shift;
	$MasterSites->siteInfo($self->{site})->getSiteHash();
}

sub getCommInfo {
	my $self = shift;
	$MasterSites->siteInfo($self->{site})->getComm();
}

sub getFileInfo {
	my $self = shift;
	my $site = $self->{site};
	my $client = $self->{client};
	my $type = $MasterSites->siteInfo($self->{site})->getRxType();
	my $model = $MasterSites->siteInfo($self->{site})->getRxModel();
	my $fileinfo = {
		filetemplate => $CONFIG->getFileTemplate($client,$site,$type,$model),
#		filetemplate => "$self->{site}_+G_+g_%y_%j_+s.raw"
	};
}

sub getTaskSpans {
	my $self = shift;
	$Tasks->getTaskSpans($self->{site}, $DB,  @_);
}

sub getSiteSessions {
	my $self = shift;
	[ $MasterSites->siteInfo($self->{site})->getSessions() ];
}

sub getAllJobInfo {
	my $self = shift;

	my $tmp = {
		download => $self->getTaskSpans("download"),
		delete => $self->getTaskSpans("delete"),
		configure => $self->getTaskSpans("configure"),
	};
	return(
		$self->getSiteCode(),
		$self->getPortInfo(),
		$self->getSiteInfo(),
		$self->getCommInfo(),
		$self->getFileInfo(),
		$tmp,
		$self->getSiteSessions()
	);
}


1;
