package Tasks;

use Debug qw(debug addToSumFile);
use Data::Dumper;
use strict;
use Exporter;
use XML::Parser;
use GPSSite;
use MyTasks;

use vars qw(@ISA $VERSION $AUTOLOAD);

@ISA = qw(Exporter);

$VERSION = do {my @r=(q$Revision: 1.1 $=~/\d+/g);sprintf "%d."."%02d"x$#r,@r};

my $currentsite = "";
my @currentblock;
my $tasks = {};

sub new {
	my $class = shift;
	my $file  = shift;

	debug(50,"Tasks::new Entering...\n");
	debug(55,"Tasks::new Reading file $file\n");
	my $self = bless {}, $class;

	# Load the file.
	my $p1 = new XML::Parser(Style => 'Subs');
	my $tree = $p1->parsefile($file) or do {
		$@ = "EGADS::Lists::Tasks cannot open `$file' for reading: $!\n";
		return;
	};

	$self->{file} = $file;
	$self->{tasks} = $tasks;

	debug(50,"Tasks::new Exiting...\n");
	$self;
}

sub isActiveTask {
	my $self = shift;
	my $site = shift;

	return(0) if ($self->{tasks}->{$site}->{active} eq "no");
	return(1);

}

sub isTasks {
	my $self = shift;
	my $site = shift;

	return(-1) unless ($self->{tasks}->{$site});

	return(0) unless ($self->isActiveTask($site) > 0);

	my @keys = keys(%{$self->{tasks}->{$site}->{tasks}});
	return(0) unless ($#keys >= 0);

	return(1);
}

sub getSiteTaskID {
	my $self = shift;
	my $site = shift;

	return(undef) if ($self->isTasks($site) <= 0);

	if ($self->{tasks}->{$site}->{tasks}) {
		return(keys(%{$self->{tasks}->{$site}->{tasks}}));
	}
	return(undef);
}

sub getSiteTaskbyID {
	my $self = shift;
	my $site = shift;
	my $id = shift;

	return(undef) if ($self->isTasks($site) <= 0);

	if ($self->{tasks}->{$site}->{tasks}->{$id}) {
		return($self->{tasks}->{$site}->{tasks}->{$id});
	}
	return(undef);
}

sub site {
	my ($xp, $el, %att) = @_;
	debug(50,"Tasks::site: Entering...\n");
	die "In the middle of site block" if ($currentsite ne "");
	die "Site already loaded"         if ($tasks->{$att{code}});

	$currentsite = $att{code};
	$tasks->{$currentsite}->{code} = $att{code};
	$tasks->{$currentsite}->{active} = $att{active};
	$tasks->{$currentsite}->{inherit} = $att{inherit};
	if ($att{inherit} ne "") {
		my $parent = $att{inherit};
		foreach my $id (keys(%{$tasks->{$parent}->{tasks}})) {
			task(1, 1, id => $id, %{$tasks->{$parent}->{tasks}->{$id}}) ;
		}
	}
	debug(50,"Tasks::site: Exiting...\n");
	1;
}

sub site_ {
	my ($xp, $el, %att) = @_;
	$currentsite = "";

}

sub task {
	my ($xp, $el, %att) = @_;
	debug(50,"task: Entering...\n");
	die "Not in site block" if ($currentsite eq "");

	my $id = $att{id};
	$tasks->{$currentsite}->{tasks}->{$id}->{frequency} = eval $att{frequency};
	$tasks->{$currentsite}->{tasks}->{$id}->{task} = $att{task};
	$tasks->{$currentsite}->{tasks}->{$id}->{delay} = eval $att{delay};
	$tasks->{$currentsite}->{tasks}->{$id}->{back} = eval $att{back};
	$tasks->{$currentsite}->{tasks}->{$id}->{time} = eval $att{time};
	$tasks->{$currentsite}->{tasks}->{$id}->{args} = $att{args};

	debug(50,"task: Exiting...\n");
	1;
}

sub returnVal {
	my $self = shift;
	my $sitecode = shift;
	my $DB = shift;
	my $action = shift;
	my @values = @_;

	my $taskid = $self->getTaskID($sitecode,$action);
	return(undef) unless(defined($taskid));
	$action =~ tr/A-Z/a-z/;
	my $class = "Tasks::Action::$action";
	my $args = $self->{tasks}->{$sitecode}->{tasks}->{$taskid}->{args};
	my $tasktype = $class->new(
				DB => \$DB,
				sitecode => $sitecode,
				taskid => $taskid,
				args => $args,
				Tasks => \$self);
	my $tmp = $tasktype->returnVal(@values);
	debug(40, "Tasks::returnVal: $sitecode $tasktype\n");
	
	return($tmp);

}

sub getTaskSpans {
	my $self = shift;
	my $sitecode = shift;
	my $DB = shift;
	my $action = shift;

	my $taskid = $self->getTaskID($sitecode,$action);
	return(undef) unless(defined($taskid));
	$action =~ tr/A-Z/a-z/;
	my $class = "Tasks::Action::$action";
	my $args = $self->{tasks}->{$sitecode}->{tasks}->{$taskid}->{args};
	my $tasktype = $class->new(
				DB => \$DB,
				sitecode => $sitecode,
				taskid => $taskid,
				args => $args,
				Tasks => \$self);
	my $tmp = $tasktype->editSpans($self->{$sitecode}->{spans}->{$action});
	debug(40, "Tasks::getTaskSpans: $sitecode $tasktype\n");
	
	return($tmp);
}

sub getTaskID {
	my $self = shift;
	my $sitecode = shift;
	my $tasktype = shift;

	debug(50,"Tasks::getTaskID: Entering...\n");
	debug(25,"sitecode = $sitecode, tasktype = $tasktype\n");

	foreach my $id ( keys(%{$self->{tasks}->{$sitecode}->{tasks}}) ) {
		my $tmptask = $self->{tasks}->{$sitecode}->{tasks}->{$id}->{task};
		return($id) if ($tmptask eq $tasktype);
	}
	debug(50,"Tasks::getTaskID: Exiting...\n");
	return(undef);

}

sub createSpans {
	my $self = shift;
	my $sitecode = shift;
	my $DB = shift;
	my $action;

	debug(50,"Tasks::createSpans: Entering...\n");
	$self->{$sitecode}->{spans} = {};
	my @tasks = $self->getSiteTaskID($sitecode);
	debug(10, "Tasks::createSpans: Creating spans for " . ($#tasks+1) . " task(s)\n");
	my $locktime = 0;
	foreach my $taskid (@tasks) {
		my $taskinfo = $self->getSiteTaskbyID($sitecode, $taskid);
		next unless ($taskinfo);
		my $time = time();
		my $action = $taskinfo->{task};
		debug(25,"Tasks::createSpans: Checking $action for Open Spans\n");
		next unless $DB->isOpenSpan( $sitecode,
			 $taskid,
			 ($time - $taskinfo->{delay}) - $taskinfo->{frequency},
			 $time - $taskinfo->{back});
		debug(25,"Tasks::createSpans: Scheduling $action task for site " . $sitecode . ", task id = " . $taskid . "\n");

		my @spansToCheck = $self->getSpansToCheck($taskinfo);
		debug(25,"Tasks::createSpans: Returned $#spansToCheck spans to check\n");
		my $maxendDt = 0;
		foreach my $spanRef (@spansToCheck) {
			my $startDt = $spanRef->{start_time};
			my $endDt = $spanRef->{stop_time};
			my $session = $spanRef->{session};
			$maxendDt = $endDt if ($endDt > $maxendDt);
			my $args = $self->{tasks}->{$sitecode}->{tasks}->{$taskid}->{args};
			$action =~ tr/A-Z/a-z/;
			my $class = "Tasks::Action::$action";
			my $tasktype = $class->new(
					DB => \$DB,
					sitecode => $sitecode,
					taskid => $taskid,
					args => $args,
					Tasks => \$self);
			debug(101,"Tasks::createSpans: Checking Span $startDt => $endDt session = $session\n");
			my $tmp = $tasktype->isSpanOK($startDt, $endDt);
			if ($tmp) {
				$locktime += $taskinfo->{time};
				push(@{$self->{$sitecode}->{spans}->{$action}}, $spanRef);
			}
		}
	}

	debug(50,"Tasks::createSpans: Exiting (locktime = $locktime)...\n");
	return($locktime);
}


# Generate a list of Spans to check for download status based on the FAT file, 
# if possible.  If not, program as MasterSites "session(s)" indicates for the last 
# DAYS_AGO days.
sub getSpansToCheck {
	my $self = shift;
	my $taskinfo = shift;

	my $frequency = $taskinfo->{frequency};
	my $delay = $taskinfo->{delay};
	my $back = $taskinfo->{back};

	debug(25,"Tasks::getSpansToCheck: Starting span check from " . gmtime(time() - $back) . "\n");
	my @spansToCheck;

	# Setup the default span of days from the DAYS_AGO config var
	my $dtEnd = time();

	# Setup the YJ-level DT objects to loop through
	my $dtStart = $dtEnd - $back;
	#Set to beginning of day
	my $start_time = $dtStart - ($dtStart%86400);
	my $stop_time = $start_time + 86400;


	my $session = 0;
	debug(75,"Tasks::getSpansToCheck: frequency = $frequency (should be positive\n");
	while (($start_time + $frequency) < $dtStart) {
		$start_time += $frequency;
		$session++;
	}

	debug(25,"Tasks::getSpansToCheck: Comparing " . ($start_time + $frequency + $delay) . " with " . ($dtEnd) . "\n");
	debug(75,"Tasks::getSpansToCheck:2: frequency = $frequency (should be positive\n");
	while (($start_time + $frequency + $delay) <= $dtEnd) {

		my $sesschar = chr(ord("a")+$session);
		$session++;
		my $span = $frequency;
		if (($start_time + $frequency) >= $stop_time) {
			$span = $stop_time - $start_time;
			$stop_time += 86400;
			$session = 0;
		}

		my $tmp = {};
		$tmp->{start_time} = $start_time;
		$tmp->{stop_time} = $start_time + $span - 1;
		$tmp->{session} = $sesschar;

		$start_time += $span;

		push(@spansToCheck,$tmp);
	}
	
	return @spansToCheck;
}

1;

